using System;
using System.Text;

namespace Hl7Service
{
    enum Hl7State
    {
        NEED_TO_POLL_APP,
        WAITING_FOR_ACK
    }

    class ClientState
    {
        private string messageId;
        private string message;
        // The time we expect an ACK by
        private DateTime ackEta;
        private Hl7State currentState;
        private Random random = new Random();

	public string MessageId
	{
		get { return messageId; }
	}

	public string Message
	{
		get { return message; }
	}

	public DateTime AckEta
	{
		get { return ackEta; }
	}

	public Hl7State CurrentState
	{
		get { return currentState; }
	}

        private string MakeMessageId()
        {
            DateTime CurrentTime = System.DateTime.Now;
            return CurrentTime.Hour.ToString() + CurrentTime.Minute.ToString() + CurrentTime.Second.ToString() + CurrentTime.Millisecond.ToString();
        }

        public bool FetchMessage()
        {
            if (random.Next(2) != 1)
            {
                // We only have a message one time in 2
                return false;
            }

            messageId = MakeMessageId();
            message = "MSH|^~\\&|MESA_ADT|XYZ_ADMITTING|MESA_IS|XYZ_HOSPITAL|" + messageId + "||ADT^A04|" + MessageId + "|P|2.3.1||||||||\r" +
                      "EVN||200004211000||||200004210950\r" +
                      "PID|||583020^^^ADT1||REYNOLDS^MALCOLM||19710321|M||AI|SERENITY^^FIREFLY CLASS CARGO SHIP|||||||20-98-1701||||||||||||\r";
            return true;
        }

        public void ConfirmMessageSend()
        {
            // TODO - a real application would mark the current message as sent in it's database.
            currentState = Hl7State.NEED_TO_POLL_APP;
        }

        public void SetAckTimeOut(int timeoutInSeconds)
        {
            ackEta = DateTime.Now;
            ackEta.AddSeconds(timeoutInSeconds);
            currentState = Hl7State.WAITING_FOR_ACK;
        }

        public bool IsTimedOut()
        {
            return (DateTime.Now > AckEta);
        }
    }


}
