using System.ServiceProcess;
using System.Text;
using System;

// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FIT-
// NESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
// iNTERFACEWARE BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
// IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CON-
// NECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

// This example code is merely intended to show how one can write a Service program
// using the LLP components supplied with Chameleon.
// This example:
// 1) Does not perform any real processing of the incoming messagess.
// 2) Does not truncate the log file hl7log.txt that the service produces in its directory.  
//    A real application should have some log purging strategy.
// 3) Does not use a configuration file.  A real application should have these configuration parameters configurable.
// 4) There is no administration interface for this service.  
//
// Before investing time and money in developing all of the above it might be prudent to consider the option of using
// Iguana instead which does all of the above and more.  See:
//
//    http://www.interfaceware.com/iguana.html
//
// To test the service it is easiest to run it as Hl7Service --debug from the command line.
// This will mean it runs as a console application.
// 
// To install it as a service program you will need to execute InstallUtil Hl7Service.EXE
// using the InstallUtil command program that comes with your .NET SDK

namespace Hl7Service
{
   class Program
   {
      /// <summary>
      /// The main entry point for the application.
      /// </summary>
      static void Main(string[] args)
      {
          bool isDebugging = false;

          foreach (string arg in args)
          {
              if (arg == "--debug")
              {
                  Console.WriteLine("Running in command line debug mode.");
                  isDebugging = true;
              }
              else
              {
                  Console.WriteLine("Unknown command line argument: " + arg);
                  Console.WriteLine("To run this service in command line mode use --debug");
                  return;
              }
          }
          if (!isDebugging)
          {
              ServiceBase[] servicesToRun;
              servicesToRun = new ServiceBase[] { new ServiceModule() };
              ServiceBase.Run(servicesToRun);
          }
          else
          {
              ServiceModule service = new ServiceModule();
              service.OnStartImp(false);
          }
      }
   }
}